<?php
require_once("cwfUI.inc.php");

class ViewDataUI extends cwfUI {
  protected static $accessRight = "";
  protected static $dataClass = "";
  protected $view;
  protected $numRows;
  protected $eventId;
  
  protected $columns;
  protected $sql;
  protected $showSelect;
  protected $showSelectAll;
  protected $disableSort;
  protected $disableFilter;

  /* 
   * Filtervariable übergeben (default: $_SESSION[...])
   * View übergeben
   * Default-Filter in View
   * Default-Order-By in View
   * Additional-Rows-Variable
   * Additional-Filter-Variable
   * Display-Checkbox-Edit-Flag
   * => GenericViewUI (=jetzige ViewDataUI?), die von anderen Klassen instanziiert wird?
   */
  
  /////////////////////////////////////////////////////////////////////////////////////////////////
  // initialize
  public function __construct(DbConnection $DB) {
    parent::__construct();
    $this->initialize($DB);

    if(!isset($_POST['action']) && isset($_POST['kdb_send_email']) && $_POST['kdb_send_email']) {
      $this->sendEmail($DB);
    }
  }
  
  protected function initialize(DbConnection $DB) {
    parent::initialize();    
    $this->eventId = null;
    $this->columns = array();
    $this->showSelect = true;
    $this->showSelectAll = true;
    $this->disableSort = false;
    $this->disableFilter = false;
    
    if(isset($_SESSION['event']))
      $this->eventId = $_SESSION['event']->event_id;
    
    if(isset($_SESSION['user']) && !isset($_SESSION['registration']['booking_id']) && isset($_SESSION['event']) && $_SESSION['event']->allow_multiple_reg) {
      $this->view = new View();
      $this->view->columns = array("booking_date", "booked_options", "grandtotal");
      $this->view->edit_booking_check = 0;
      $this->view->edit_user_data = 0;
      $this->view->where_clause="u.user_id='{$_SESSION['user']->user_id}'";
      $this->view->sort_column = "booking_date";
      $this->view->sort_desc = 1;
      $this->showSelect = false;
      $this->showSelectAll = false;
      $this->disableFilter = true;
    }
    else if(isset($_SESSION['view_id']) && $_SESSION['view_id']) {
      $this->view = new View();
      $this->view->readData($DB, "view_id={$_SESSION['view_id']}");
      $this->title = $this->view->view_name;
      if($_SESSION['admin']['login']!="root" &&
         count($this->view->db_admin_ids)>0 && !in_array($_SESSION['admin']['db_admin_id'], $this->view->db_admin_ids)) {
        $this->view = null;
        $this->error = _("You do not have enough privileges.");
        return;
      }
    }
    else {
      $this->error=_("No view selected");
      return;
    }
    
    if(isset($_POST))
      Plugins::viewButtonPushed($DB);
    
    $this->sql = $this->getSQL($DB);
    
    $this->header = '  <script src="'.CWF_HTTP_PATH.'k_db/js/kdb_view.js" type="text/javascript"></script>'."\n";
    $this->header .= '  <link rel="stylesheet" href="'.CWF_HTTP_PATH.'k_db/css/view.css" type="text/css">';
    $this->header .= "  <script type=\"text/javascript\">
    var uiClassName = '".get_class($this)."';
    
    $(document).ready(function(){
      setupTableEvents(1);
      setupTableFixedHeader();\n";
    if($this->view->edit_user_data)
      $this->header .= "      $(\"#kdb_form_user\").validate({
        errorClass: 'invalid',\n".
            static::getValidationRules('User')->getValidationRulesJQuery("        ")."
      });\n";
    if($this->view->edit_booking_check)
      $this->header .= "      $(\"#kdb_form\").validate({
        errorClass: 'invalid',\n".
            static::getValidationRules('UserBookingCheck')->getValidationRulesJQuery("        ")."
      });\n";
      $hiddenColumns = array();
      foreach($this->columns as $key => $value)
        if(in_array($value['columnName'], $this->view->hidden_columns) || in_array($value['columnType'], $this->view->hidden_columns))
          $hiddenColumns[] = $key+1;
      $this->header .= "      viewId = window.location.search.match(/view_id=([0-9]+)/);
      if(viewId) {
        viewId = viewId[1];
        hiddenColumns = sessionStorage.getItem('hiddenColumns_'+viewId);
        if (!hiddenColumns) {
          hiddenColumns = '".implode(";", $hiddenColumns)."';
        }
        if(hiddenColumns) {
          $.each(hiddenColumns.split(';'), function(key, value) {
            kdbHideColumn(value, $('#kdb_hide_col_'+value));
          });
        }
      }\n";
    $this->header .= "    });\n";
    $this->header .= "  </script>\n";
  }

  /////////////////////////////////////////////////////////////////////////////////////////////////
  // security
  static public function isAllowedAction($action) {
    if (parent::isAllowedAction($action))
      return true;
    
    return in_array($action, array("updateView", "merge", "sendEmail", "loadTemplate", "saveTemplate", "deleteTemplate"));
  }
  
  /////////////////////////////////////////////////////////////////////////////////////////////////
  // handle data
  public function getIDName() {
    return "";
  }
  
  public function getId() {
    return "";
  }
  
  static public function getValidationRules($form=null) {
    $rules = array();
    $messages = array();
    switch($form) {
      case "UserBookingCheck":
        break;
      case "User":
        $rules = array(
            "email"=>array("email"=>true, "remote"=>array("url"=>CWF_HTTP_PATH."admin/checkValue.php", "data"=>array("class"=>"User", "value"=>"#email", "id"=>"#user_id"))), 
            "date_of_birth"=>array("date_".getenv("LC_ALL")=>true)
        );
        $messages = array(
            "email"=>array("email"=>_("Please enter a valid email address (e.g. you@yourdomain.com)"),
                           "remote"=>_("Email address is already taken. Please enter a different address.")),
            "date_of_birth"=>array("date_".getenv("LC_ALL")=>_("Please enter a valid date, e.g. ").CWFTools::dateISO2Local("2000-12-31", getenv("LC_ALL")))
        );
        break;
      case "EmailTemplate":
        $rules = array(
            "email_template_name"=>array("required"=>true) 
        );
        $messages = array(
            "email_template_name"=>_("Please enter a name for the template.") 
        );
        break;
      case "UserBooking":
        break;
    }
    
    $validationRules = new FormValidation($rules, $messages);
    return $validationRules;
  }
  
  public function isValid($action) {
    if(in_array($action, array("updateForm", "delete", "updateView", "sendEmail", "merge", "loadTemplate", "saveTemplate", "deleteTemplate")))
      return true;
    return $this->data->validation->isValid();
  }
  
  protected function getColumns($type, $skipEmpties=false) {
    $columns = array();
    foreach ($this->columns as $column)
      if(!$skipEmpties || $column[$type])
        $columns[] = $column[$type];
    return $columns;
  }
  
  public function getPostBackXML(DbConnection $DB, $action) {
    $xml = "";
    switch ($action) {
      case "updateView":
        if(isset($_POST['email']) && $_POST['email']) {
          $allowEmail = $DB->result($DB->query($this->getSQLEmailWarning()), 0, 0);
          $xml .= "  <replaceHTMLElement id=\"div#dialog_email #email_warning\"><![CDATA[\n";
          if($allowEmail==0)
            $xml .= "<p id=\"email_warning\"></p>";
          else
            $xml .= "<p id=\"email_warning\">"._("Some of the selected recipients DO NOT want to receive promotional emails!")."</p>";
          $xml .= "]]>\n";
          $xml .= "  </replaceHTMLElement>\n";
        }
      case "saveasnew":
      case "save":
      case "merge":
      case "delete":
      case "sendEmail":
        $xml .= "  <replaceHTMLElement id=\"table#view tbody\"><![CDATA[\n";
        $xml .= $this->getViewTableBody($DB, $this->sql);
        $xml .= "]]>\n";
        $xml .= "  </replaceHTMLElement>\n";
        $xml .= "  <replaceHTMLElement id=\"#pagination\"><![CDATA[\n";
        $xml .= $this->resultPagination();
        $xml .= "]]>\n";
        $xml .= "  </replaceHTMLElement>\n";
        break;
      case "saveTemplate":
      case "deleteTemplate":
        $xml .= "  <newValues>\n";
        $xml .= "    <newValue name=\"email_template_name\"></newValue>\n";
        $xml .= "  </newValues>\n";
        $xml .= "  <replaceHTMLElement id=\"#email_template_id\"><![CDATA[\n";
        $xml .= $this->getEmailTemplateListHTML($DB);
        $xml .= "]]>\n";
        $xml .= "  </replaceHTMLElement>\n";
        break;
      case "loadTemplate":
        $template = new EmailTemplate(self::getValidationRules("EmailTemplate"));
        $template->readData($DB, array("email_template_id"=>$_POST['email_template_id']));
        $xml .= "  <newValues>\n";
        $xml .= "    <newValue name=\"email_subject\">{$template->email_subject}</newValue>\n";
        $xml .= "    <newValue name=\"email_text\">{$template->email_text}</newValue>\n";
        $xml .= "  </newValues>\n";
        break;
      default:
        $xml = parent::getPostBackXML($DB, $action);
    }
    return $xml;
  }
  
  public function doAction(DbConnection $DB, $action) {
    $id = "";
    switch ($action) {
      case "updateForm":
      case "saveasnew":
      case "save":
        $id = parent::doAction($DB, $action, $_POST);
        break;
      case "merge":
        $this->saveForm($DB);
        $id = $this->getId();
        break;
      case "delete":
        list($errno, $error) = $this->deleteData($DB, "");
        if ($errno>0)
          $errorMsg = _("Error")." {$errno}: {$error}";
        break;
      case "sendEmail":
        $this->sendEmail($DB);
      case "updateView":
        $id = null;
        break;
      case "saveTemplate":
        unset($_POST['email_template_id']);
        $template = new EmailTemplate(self::getValidationRules("EmailTemplate"));
        $template->fromArray($_POST);
        $template->writeData($DB);
        break;
      case "deleteTemplate":
        EmailTemplate::deleteData($DB, "email_template_id='{$_POST['email_template_id']}'");
        break;
      case "loadTemplate":
        break;
    }
    return $id;
  }
  
  /////////////////////////////////////////////////////////////////////////////////////////////////
  // Database operations
  public function saveForm(DbConnection $DB) {
    if($_POST['action']=="merge") {
      $userIds = array_values(array_unique($_POST['select_user_id']));
      list($errno, $error) = User::mergeUsers($DB, $userIds, $_POST['merge_with_id']);
      if($errno) {
        $this->error = $error;
        return 0;
      }
      else {
        Plugins::mergeUsers($DB, $userIds, $_POST['merge_with_id']);
      }
      return 1;  
    }

    switch ($_POST['formName']) {
      case "kdb_form_user":
        $this->data = new User(self::getValidationRules("User"));
        break;
      case "kdb_form_booking":
        if (isset($_SESSION['event'])) {
          $this->data = new UserBooking(self::getValidationRules("UserBooking"));
          $this->data->setEventId($DB, $_SESSION['event']->event_id);
        }
        else {
          $this->error = _("No event selected.");
          return 0;
        }
        break;
      case "kdb_form_view":
        if (isset($_SESSION['event'])) {
          foreach ($_POST['booking_id'] as $idx => $value) {
            $this->data = new UserBookingCheck(self::getValidationRules("UserBookingCheck"));
            if($this->data->fromArray($_POST, $idx)) {
              list($errno, $error) = $this->data->writeData($DB);
              if ($errno) 
                $this->error = $error;
            }
          }
          return 1;
        }
        else {
          $this->error = _("No event selected.");
          return 0;
        }
        break;
    }

    if($this->data->fromArray($_POST, true)) {
      list($errno, $error) = $this->data->writeData($DB, true);
      if ($errno) {
        $this->error = $error;
        return 0;
      }
      return 1;
    }
    $this->error = _("Validation failed.");
    return 0;
  }
  
  public function deleteData(DbConnection $DB, $id) {
    if(isset($_SESSION['event'])) {
      $bookingIds = array_keys($_POST['select_user_id']);
      UserBooking::deleteData($DB, "booking_id IN ('".implode("', '", $bookingIds)."') AND event_id='{$_SESSION['event']->id}'");
    }
    else if($_SESSION['admin']['login']=="root") {
      $userIds = array_values($_POST['select_user_id']);
      User::deleteData($DB, "user_id IN ('".implode("', '", $userIds)."')");
    }
  }
  
  public function sendEmail(DbConnection $DB) {
    $attachment = array();
    if ($_FILES['email_attachment']['tmp_name']) {
      $attachment['name'] = $_FILES['email_attachment']['name'];
      if ($_FILES['email_attachment']['type'])
        $attachment['type'] = $_FILES['email_attachment']['type'];
      else
        $attachment['type'] = "application/octet-stream";
    
      $attachmentFile = fopen($_FILES['email_attachment']['tmp_name'], 'rb');
      $attachment['data'] = fread($attachmentFile, filesize($_FILES['email_attachment']['tmp_name']));
      fclose($attachmentFile);
      $attachment['data'] = chunk_split(base64_encode($attachment['data']));
    }
    
    $recipients = array();
    if(!isset($_SESSION['event']))
      $emailFrom = ROOT_EMAIL;
    else 
      $emailFrom = $_SESSION['event']->event_email;
    foreach ($_POST['select_user_id'] as $userId) {
      set_time_limit(30);
      $user = new User();
      $user->readData($DB, array("user_id"=>"'{$userId}'")); 
      $emailTxt = str_replace(array('$salutation', '$familyname$', '$givenname$'), 
                              array($user->getText($DB, 'salutation'), $user->familyname, $user->givenname),
                              $_POST['email_text']);
      CWFTools::sendEmailWithAttachment($user->email, $emailTxt, $_POST['email_subject'], $emailFrom, $attachment);
      
      $recipients[] = "{$user->familyname}, {$user->givenname} ({$user->email})";
    }
    
    $emailTxt = _("Recipients") . ":\n\n" . implode("\n", $recipients) . "\n\n\n". _("Message") . ":\n\n" . $_POST['email_text'] . "\n";
    CWFTools::sendEmailWithAttachment($emailFrom, $emailTxt, 
                                      (isset($_SESSION['event']) ? $_SESSION['event']->event_name : ORGANISATION_NAME).": ".$_POST['email_subject'], 
                                      $emailFrom, $attachment);
  }
  
  /////////////////////////////////////////////////////////////////////////////////////////////////
  // SQL preparation
  protected function prepareSQLSelectedIds() {
    $whereClauseSelect = array();
    if(isset($_POST['select_booking_id']) && count($_POST['select_booking_id'])>0) {
      $bookingIds = array_keys($_POST['select_booking_id']);
      $whereClauseSelect[] = "ub.booking_id IN (".implode(",", $bookingIds).")";
    }
    if(isset($_POST['select_user_id']) && count($_POST['select_user_id'])>0) {
      $userIds = array_values($_POST['select_user_id']);
      if($this->eventId) {
        $bookingIds = array_keys($_POST['select_user_id']);
        $whereClauseSelect[] = "ub.booking_id IN (".implode(",", $bookingIds).")";
      }
      else {
        $userIds = array_values($_POST['select_user_id']);
        $whereClauseSelect[] = "u.user_id IN (".implode(",", $userIds).")";
      }
    }
    
    return $whereClauseSelect;
  }
  
  protected function prepareSQL(DbConnection $DB) {
    if(isset($_POST['sortby'])) {
      if(isset($_POST['sortdirection']) && in_array($_POST['sortdirection'], array('DESC', 'ASC'))) {
        $_SESSION['sortby']['column'] = $_POST['sortby'];
        $_SESSION['sortby']['direction'] = $_POST['sortdirection'];
      }
      else if(isset($_SESSION['sortby']) && $_SESSION['sortby']['column']==$_POST['sortby'] && $_SESSION['sortby']['direction']!="DESC") {
        $_SESSION['sortby']['direction'] = "DESC";
      }
      else {
        $_SESSION['sortby']['column'] = $_POST['sortby'];
        $_SESSION['sortby']['direction'] = "ASC";
      }
      $_POST['sortdirection'] = $_SESSION['sortby']['direction'];
    }
    else if(isset($_POST['filter'])) {
      unset($_SESSION['filter']);
      foreach($_POST['filter'] as $key => $value) {
        if($value!="")
          $_SESSION['filter'][$key] = $value;
      }
    }
    else if(isset($_POST['limit']) && preg_match('~^[0-9]+$~', $_POST['limit'])) {
      $_SESSION['limit'] = $_POST['limit'];
    }
    
    $this->columns = array();
    $columnIdx = 0;
    if($this->showSelect) {
      $this->columns[$columnIdx]['columnName'] = "selectUser";
      $this->columns[$columnIdx]['displayName'] = "";
      if(in_array("user_id", $this->view->columns))
        $this->columns[$columnIdx++]['forSelect'] = "ub.booking_id";
      else
        $this->columns[$columnIdx++]['forSelect'] = "u.user_id, ub.booking_id";
    }
    if(!$this->eventId) {
      $this->columns[$columnIdx]['columnName'] = "event";
      $this->columns[$columnIdx]['displayName'] = _("Event");
      $this->columns[$columnIdx++]['forSelect'] = "ifnull(group_concat(event_name ORDER BY event_name SEPARATOR '<br>'), '') event";
    }
    foreach ($this->view->columns as $column) {
      if($displayName=User::getProperties($column)) {
        $this->columns[$columnIdx]['columnName'] = $column;
        $this->columns[$columnIdx]['displayName'] = $displayName;
        $this->columns[$columnIdx++]['forSelect'] = User::getPropertySelect($column, "u.");
      }
      else if($this->eventId && $displayName=UserBookingCheck::getProperties($column)) {
        $this->columns[$columnIdx]['columnName'] = $column;
        $this->columns[$columnIdx]['displayName'] = $displayName;
        $this->columns[$columnIdx++]['forSelect'] = UserBookingCheck::getPropertySelect($column, $this->view->edit_booking_check, "ubc.");
      }
      else if($this->eventId && $column=="booking_date") {
        $this->columns[$columnIdx]['columnName'] = $column;
        $this->columns[$columnIdx]['displayName'] = _("Booking date");
        $this->columns[$columnIdx++]['forSelect'] = "ub.booking_date booking_date";
      }
      else if($this->eventId && $column=="grandtotal") {
        $this->columns[$columnIdx]['columnName'] = $column;
        $this->columns[$columnIdx]['displayName'] = _("Grand total");
        $this->columns[$columnIdx++]['forSelect'] = "(SELECT sum(ef.event_fee) ".
            "-ifnull((SELECT discount FROM user_booking_check ubc WHERE ubc.booking_id=ubo.booking_id), 0) ".
            "FROM user_booking_options ubo ".
            "JOIN event_fees ef ON ef.event_fee_id=ubo.event_fee_id ".
            "WHERE ubo.booking_id=ub.booking_id ".
            "AND ubo.waitlist=0) grandtotal";
      }
      else if($this->eventId && $column=="booked_options") {
        $eventForm = new EventForm();
        $eventForm->setEventId($this->eventId);
        $eventForm->readData($DB);
        foreach ($eventForm->eventForm as $group) {
          switch($group->group_type) {
            case OPTION_GROUP_FEE:
              $this->columns[$columnIdx]['columnType'] = "booked_options";
              $this->columns[$columnIdx]['columnName'] = "event_option_group_{$group->event_option_group_id}";
              $this->columns[$columnIdx]['displayName'] = $group->group_name;
              $this->columns[$columnIdx]['forSelect'] = "(SELECT group_concat(eo.event_option ORDER BY eo.event_option_id SEPARATOR ', ') ".
                  "FROM user_booking_options ubo ".
                    "JOIN event_options eo ON eo.event_option_id=ubo.event_option_id ".
                    "JOIN event_option_groups eog ON eog.event_option_group_id=eo.event_option_group_id ".
                  "WHERE ubo.booking_id=ub.booking_id ".
                    "AND eog.event_option_group_id='{$group->event_option_group_id}') {$this->columns[$columnIdx]['columnName']},\n".
                  "(SELECT group_concat(ubo.waitlist ORDER BY eo.event_option_id SEPARATOR ', ') ".
                   "FROM user_booking_options ubo ".
                     "JOIN event_options eo ON eo.event_option_id=ubo.event_option_id ".
                     "JOIN event_option_groups eog ON eog.event_option_group_id=eo.event_option_group_id ".
                   "WHERE ubo.booking_id=ub.booking_id ".
                     "AND eog.event_option_group_id='{$group->event_option_group_id}') waitlist_{$this->columns[$columnIdx]['columnName']}";
              $columnIdx++;
              break;
            case OPTION_GROUP_ADDINFO:
              foreach ($group->eventOptions as $option) {
                switch($option->option_type) {
                  case EVENT_OPTION_TEXT:
                  case EVENT_OPTION_TEXT_MULTILINE:
                    $valueName = "value_str";
                    break;
                  case EVENT_OPTION_DATE:
                    $valueName = "value_date";
                    break;
                  case EVENT_OPTION_NUMBER_INT:
                  case EVENT_OPTION_NUMBER_DEC:
                    $valueName = "value_num";
                    break;
                }
                $this->columns[$columnIdx]['columnType'] = "booked_options";
                $this->columns[$columnIdx]['columnName'] = "event_option_{$option->event_option_id}";
                $this->columns[$columnIdx]['displayName'] = $option->event_option;
                $this->columns[$columnIdx]['forSelect'] = "(SELECT ubo.{$valueName} ".
                    "FROM user_booking_options ubo ".
                    "WHERE ubo.booking_id=ub.booking_id ".
                    "AND ubo.event_option_id='{$option->event_option_id}') {$this->columns[$columnIdx]['columnName']}";
                $columnIdx++;
              }
              break;
          }
        }
      }
      else {
        $columnIdx = Plugins::addViewSQL($this->columns, $columnIdx, $column);
      }
    }

    // Filtering
    $whereClause = array();
    if(isset($_SESSION['filter'])) {
      foreach($_SESSION['filter'] as $filterKey => $filter) {
        if(in_array($filterKey, $this->getColumns("columnName", true))) {
          if(preg_match('~^(&lt;&gt;|=|&lt;=|&gt;=|&gt;|&lt;)(.*)$~', $filter, $match)) {
            CWFTools::decodeHTMLChars($match[1]);
            if(in_array($match[1], array("=", "<>")) && strtolower($match[2])=='null')
              $whereClause[] = $filterKey.($match[1]=="=" ? " IS NULL" : " IS NOT NULL");
            else if($match[2]=='&quot;&quot;' || $match[2]=='&#039;&#039;')
              $whereClause[] = $filterKey."{$match[1]}''";
            else
              $whereClause[] = $filterKey."{$match[1]}'{$match[2]}'";
          }
          else if(preg_match('~^/(.*)/$~', $filter, $match)) {
            $whereClause[] = $filterKey." REGEXP '{$match[1]}'";
          }
          else
            $whereClause[] = $filterKey." LIKE '".str_replace(array("%", "_", "*", "?"), array("\\%", "\\_", "%", "_"), $filter)."'";
        }
      }
    }
    
    // Show only selected rows
    if(isset($_POST['kdb_selected']) && $_POST['kdb_selected']) {
      $whereClauseSelect = $this->prepareSQLSelectedIds();
      if(count($whereClauseSelect)>0)
        $whereClause[] = "(".implode(") OR (", $whereClauseSelect).")";
      $limit = "";
    }
    else {
      if(!isset($_SESSION['limit'])) $_SESSION['limit'] = 100;
      if(!isset($_POST['page']) || !preg_match('~^[0-9]+$~', $_POST['page'])) $_POST['page']=1;
      $limit = "LIMIT ".(($_POST['page']-1)*$_SESSION['limit']).", ".($_SESSION['limit']);
    }
    
    // Ordering
    if(isset($_SESSION['sortby']['column']) && in_array($_SESSION['sortby']['column'], $this->getColumns("columnName", true)))
      $orderBy = "{$_SESSION['sortby']['column']} {$_SESSION['sortby']['direction']}";
    else if($this->view->default_order_by)
      $orderBy = $this->view->default_order_by;
    else
      $orderBy = "u.user_id";
    
    return array($whereClause, $orderBy, $limit);
  }
  
  protected function getSQLEmailWarning() {
    $whereClauseSelect = $this->prepareSQLSelectedIds();
    if(count($whereClauseSelect)>0)
      $whereClause[] = "(".implode(") OR (", $whereClauseSelect).")";
    
    $sql = "SELECT count(*) allow_email\n".
        "FROM users u\n".
        "  JOIN user_booking ub ON (ub.user_id=u.user_id AND ub.deleted=0)\n".
        "  LEFT JOIN user_booking_check ubc ON (ubc.booking_id=ub.booking_id)\n".
        "WHERE u.deleted=0\n".
        "  AND u.allow_email=0\n".
        ($this->eventId ? "  AND ub.event_id='{$this->eventId}'\n" : "").
        ($this->view->where_clause ? "  AND ({$this->view->where_clause})\n" : "").
        (count($whereClause)>0 ? "  AND ".implode("\n  AND ", $whereClause)."\n" : "").";";
    
    return $sql;
  }
  
  protected function getSQL(DbConnection $DB, $columns="") {
    list($whereClause, $orderBy, $limit) = $this->prepareSQL($DB);
    
    if($this->eventId)
      $sql = "SELECT SQL_CALC_FOUND_ROWS ".implode(",\n       ", $this->getColumns("forSelect", true))."\n".
             "FROM users u\n".
             "  JOIN user_booking ub ON (ub.user_id=u.user_id AND ub.deleted=0)\n".
             "  LEFT JOIN user_booking_check ubc ON (ubc.booking_id=ub.booking_id)\n".
             "WHERE ub.event_id='{$this->eventId}'\n".
             "  AND u.deleted=0\n".
             ($this->view->where_clause ? "  AND ({$this->view->where_clause})\n" : "").
             (count($whereClause)>0 ? "HAVING ".implode("\n  AND ", $whereClause)."\n" : "").
             "ORDER BY {$orderBy} ".
             $limit.";";
    else
      $sql = "SELECT SQL_CALC_FOUND_ROWS ".implode(",\n       ", $this->getColumns("forSelect", true))."\n".
             "FROM users u\n".
             "  LEFT JOIN user_booking ub ON (ub.user_id=u.user_id AND ub.deleted=0)\n".
             "  LEFT JOIN events e ON e.event_id=ub.event_id\n".
             "WHERE u.deleted=0\n".
             ($this->view->where_clause ? "  AND ({$this->view->where_clause})\n" : "").
             "GROUP BY u.user_id\n".
             (count($whereClause)>0 ? "HAVING ".implode("\n  AND ", $whereClause)."\n" : "").
             "ORDER BY {$orderBy} ".
             $limit.";";
    
    return $sql;
  }
  
  /////////////////////////////////////////////////////////////////////////////////////////////////
  // output
  protected function getEmailTemplateListHTML(DbConnection $DB) {
    $templates = EmailTemplate::readList($DB);
    if(count($templates)==0)
      $str = "    <select id=\"email_template_id\" name=\"email_template_id\" size=\"1\" onChange=\"kdbLoadEmailTemplate('ViewDataUI', 'kdb_form_view');\" disabled>\n";
    else
      $str = "    <select id=\"email_template_id\" name=\"email_template_id\" size=\"1\" onChange=\"kdbLoadEmailTemplate('ViewDataUI', 'kdb_form_view');\">\n";
    $str .= "    <option value=\"-1\">"._("Choose template")."</option>\n";
    foreach ($templates as $template)
      $str .= "      <option value=\"{$template['id']}\">{$template['name']}</option>\n";
    $str .= "    </select>\n";
        
    return $str;
  }
  protected function resultPagination() {
    $str = "<div id=\"pagination\"><b><input type=\"text\" id=\"limit\" name=\"limit\" value=\"{$_SESSION['limit']}\" onKeyDown=\"if(event.which==13) { event.preventDefault(); kdbUpdateViewTable('".get_class($this)."', 'kdb_form_view', 'page=1&limit='+$(this).val()); return false; }\" onChange=\"kdbUpdateViewTable('".get_class($this)."', 'kdb_form_view', 'page=1&limit='+$(this).val()); }\">/{$this->numRows}</b>";
    if (isset($_SESSION['limit']) && $_SESSION['limit']<$this->numRows) {
      $str .= ", "._("Page").": ";
      for ($i=1; $i<=ceil($this->numRows/$_SESSION['limit']); $i++)
        if(isset($_POST['page']) && $_POST['page']==$i)
          $str .= "<span onClick=\"kdbUpdateViewTable('".get_class($this)."', 'kdb_form_view', 'page={$i}');\"><u>{$i}</u></span>, ";
        else
          $str .= "<span onClick=\"kdbUpdateViewTable('".get_class($this)."', 'kdb_form_view', 'page={$i}');\">{$i}</span>, ";
      
      $str = substr($str, 0, -2);
    }
    $str .= "\n  <span id=\"tt_limit\" class=\"tooltip below\">"._("Number of rows displayed per page.")."</span>\n";
    $str .= "</div>\n";
    
    return $str;
  }
  
  public function getCSV(DbConnection $DB) {
    if(!$this->eventId && $_SESSION['admin']['login']!="root") {
      return _("No event selected.")."\n";
    }
    
    $this->showSelect = false;
    $this->showSelectAll = false;
    $this->view->edit_booking_check = 0;
    $this->sql = $this->getSQL($DB);
    
    //echo "<pre>{$this->sql}</pre>\n";
    $result = $DB->query($this->sql);
    
    if($DB->error()) {
      $str = "{$DB->error()}\n";
    }
    else if ($DB->num_rows($result)==0) {
      $str = _("No data found.")."\n";
    }
    else {
      $str = implode("\t", $this->getColumns('displayName'))."\n";
      while ($row = $DB->fetch_array($result)) {
        $rowStr = array();
        foreach ($this->columns as $column) {
          if(strpos($column['forSelect'], "ubo.waitlist")!==false) {
            $values = explode(", ", $row[$column["columnName"]]);
            $waitlist = explode(", ", $row["waitlist_".$column["columnName"]]);
            $optionsStr = array();
            foreach ($values as $idx => $value) {
              if (isset($waitlist[$idx]) && $waitlist[$idx])
                $optionsStr[] = "$value (W)";
              else
                $optionsStr[] = "$value";
            }
            $rowStr[] = implode(", ", $optionsStr);
          }
          else if(isset($column['process']) && $column['process'])
            $rowStr[] = $column['process']['class']::$column['process']['function']($column["columnName"], $row[$column["columnName"]]);
          else
            $rowStr[] = "{$row[$column["columnName"]]}";
        }
        $str .= implode("\t", $rowStr)."\n";
      }
    }
    return $str;
  }
  
  protected function getColumnHeader($column) {
    $str = "";
    $filterActive = "";
    $filterValue = "";
    if(isset($_SESSION['filter'][$column['columnName']])) {
      $filterActive = " active";
      $filterValue = $_SESSION['filter'][$column['columnName']];
    }
    $sortActive = "";
    if(isset($_SESSION['sortby']['column']) && $_SESSION['sortby']['column']==$column['columnName']) {
      if($_SESSION['sortby']['direction']=="DESC")
        $sortActive = " sort_descending";
      else
        $sortActive = " sort_ascending";
    }
    else if($this->view->sort_column==$column['columnName']) {
      if($this->view->sort_desc)
        $sortActive = " sort_descending";
      else
        $sortActive = " sort_ascending";
    }
    if($column['columnName']=="selectUser") {
      if($this->showSelectAll)
        $str .= "  <th><div class=\"tableheader\"><input type=\"checkbox\" class=\"select_all\" onClick=\"kdbViewSelectAll($(this));\">&nbsp;</div></th>\n";
      else
        $str .= "  <th><div class=\"tableheader\">&nbsp;</div></th>\n";
    }
    else {
      $str .= "  <th><div class=\"tableheader\" id=\"header_{$column['columnName']}\">".
          "<span class=\"th_text{$sortActive}".($this->disableSort ? "" : " sortable")."\">{$column['displayName']}</span>".
          ($this->disableFilter ? "" : "<span class=\"showfilter{$filterActive}\" onClick=\"kdbShowTableFilter(\$(this));\"></span>".
                                       "<div class=\"filter\"><input type=\"text\" id=\"filter[{$column['columnName']}]\" name=\"filter[{$column['columnName']}]\" style=\"width:150px;\" placeholder=\""._("Filter")."\" value=\"{$filterValue}\"></div>");
      $str .= "</div></th>\n";
    }
    return $str;
  }
  
  protected function getViewTableHeader() {
    $str = "<colgroup>\n";
    foreach ($this->columns as $column) {
      $str .= "  <col id=\"col_{$column['columnName']}\">\n";
    }
    $str .= "</colgroup>\n";

    $str .= "<thead>\n";
    
    $str .= "<tr><th id=\"hideWhileScrolling\" colspan=\"".(count($this->columns))."\"><div></div></th></tr>\n";
    $str .= "<tr id=\"headerrow\">\n";

    foreach ($this->columns as $column) {
      $str .= $this->getColumnHeader($column);
    }
      
    $str .= "</tr>\n";
    $str .= "</thead>\n";
    
    return $str;
  }
  
  protected function getViewTableCell($column, $row) {
    $str = "";
    if($column['columnName']=="selectUser") {
      if($this->eventId)
        $str .= "  <td>".
                "<input type=\"checkbox\" id=\"select_user_id_{$row['booking_id']}\" name=\"select_user_id[{$row['booking_id']}]\" value=\"{$row['user_id']}\"".
                ((isset($_POST['select_user_id'][$row['booking_id']]) && $_POST['select_user_id'][$row['booking_id']]) ? " checked" : "").
                ">";
      else
        $str .= "  <td>".
                "<input type=\"checkbox\" id=\"select_user_id_{$row['user_id']}\" name=\"select_user_id[{$row['user_id']}]\" value=\"{$row['user_id']}\"".
                ((isset($_POST['select_user_id'][$row['user_id']]) && $_POST['select_user_id'][$row['user_id']]) ? " checked" : "").
                ">";
      if($this->view->edit_user_data)
        $str .= "<span class=\"edit_user\" onClick=\"kdbUpdateViewDialog('{$row['user_id']}', '{$row['booking_id']}', ".(($this->eventId!=null) ? "true" : "false").");\">&nbsp;</span>".
                "<input type=\"hidden\" id=\"booking_id[{$row['booking_id']}]\" name=\"booking_id[{$row['booking_id']}]\" value=\"{$row['booking_id']}\">";
      $str .= "</td>\n";
    }
    else if(substr($column['forSelect'], 0, 4)=="ubc." && $this->view->edit_booking_check)
      $str .= "  <td>".UserBookingCheck::getFormElement($column['columnName'], "{$column['columnName']}[{$row['booking_id']}]", $row[$column['columnName']])."</td>";
    else if(strpos($column['forSelect'], "ubo.waitlist")!==false && $column['columnName']!="grandtotal") {
      $values = explode(", ", $row[$column['columnName']]);
      $waitlist = explode(", ", $row["waitlist_".$column['columnName']]);
      $optionsStr = array();
      foreach ($values as $idx => $value) {
        if (isset($waitlist[$idx]) && $waitlist[$idx])
          $optionsStr[] = "<span class=\"waitlist\">$value</span>";
        else
          $optionsStr[] = "$value";
      }
      $str .= "  <td>".implode(", ", $optionsStr)."</td>\n";
    }
    else if(isset($column['process']) && $column['process'])
      $str .= "  <td>".$column['process']['class']::$column['process']['function']($column["columnName"], $row[$column["columnName"]])."</td>\n";
    else
      $str .= "  <td>{$row[$column['columnName']]}</td>\n";
    
    return $str;
  }
  
  protected function getViewTableBody(DbConnection $DB) {
    //echo "<pre>{$this->sql}</pre>\n";
    $result = $DB->query($this->sql);
    
    $str = "<tbody>\n";
    if($DB->error()) {
      $str .= "<tr><td colspan=\"".(count($this->columns))."\">{$DB->error()}</td></tr>\n";
    }
    else if ($DB->num_rows($result)==0) {
      $str .= "<tr><td colspan=\"".(count($this->columns))."\">"._("No data found.")."</td></tr>\n";
    }
    else {
      $this->numRows = $DB->result($DB->query("SELECT found_rows();"), 0, 0);
      while ($row = $DB->fetch_array($result)) {
        $str .= "<tr>\n";
        
        foreach ($this->columns as $column)
          $str .= $this->getViewTableCell($column, $row);

        $str .= "</tr>\n";
      }
    }
    $str .= "</tbody>\n";
    
    return $str;
  }
  
  public function getViewTable(DbConnection $DB) {
    //echo "<pre>{$this->sql}</pre>\n";
    
    $str = "<table id=\"view\" class=\"view\">\n";
    
    $str .= $this->getViewTableHeader();
    $str .= $this->getViewTableBody($DB);
    
    $str .= "</table>\n";
    return $str;
  }
  
  public function getHTML(DbConnection $DB) {
    if(!$this->view) return;
    
?>
<form id="kdb_form_view" action="" method="post" enctype="multipart/form-data">

<div class="form_controls_top">
  <div class="multibuttoncontainer">
    <input type="submit" id="kdb_hidecolumns" name="kdb_hidecolumns" value="" onClick="$(this).siblings('.multibutton').toggle(300); return false;">
      <span id="tt_kdb_hidecolumns" class="tooltip below"><?=_("Show / hide table columns.")?></span>
    <div class="multibutton">
<?php
    foreach($this->columns as $columnIdx => $column)
      if($column['columnName']!="selectUser")
        echo "      <p id=\"kdb_hide_col_".($columnIdx+1)."\" onClick=\"kdbHideColumn(".($columnIdx+1).", $(this));\">{$column['displayName']}</p>\n";
?>
  </div></div>
<p>
<?php 
    if($this->view->edit_user_data) {
?>
  <input type="submit" id="kdb_save" name="kdb_save" value="<?=_("Save")?>" onClick="kdbSaveEntry('ViewDataUI', 'save', 'kdb_form_view', refreshHiddenColumns); return false;">
  <input type="submit" id="kdb_delete_selected" name="kdb_delete" value="<?=_("Delete")?>" onClick="kdbActionConfirmation('ViewDataUI', 'kdb_form_view', 'dialog_delete'); return false;">
  <input type="submit" id="kdb_merge_selected" name="kdb_merge" value="<?=_("Merge")?>" onClick="kdbActionConfirmation('ViewDataUI', 'kdb_form_view', 'dialog_merge'); return false;">
<?php
    } 
?>
  <input type="submit" id="kdb_email" name="kdb_email" value="<?=_("Write email")?>" onClick="kdbActionConfirmation('ViewDataUI', 'kdb_form_view', 'dialog_email'); return false;">
  <input type="submit" id="kdb_csv" name="kdb_csv" value="CSV" onClick="kdbGetCSV('ViewDataUI', 'kdb_form_view'); return false;">
    <span id="tt_kdb_csv" class="tooltip below"><?=_("Get CSV")?></span>
<?php
    $newButtons = Plugins::addKDbViewButtons();
    if (count($newButtons)>0) {
      foreach($newButtons as $newButton) {
        echo "  <input type=\"submit\" id=\"{$newButton['buttonname']}\" name=\"{$newButton['buttonname']}\" value=\"{$newButton['buttontext']}\" style=\"background-image:url({$newButton['buttonicon']});\">\n";
        if($newButton['buttontooltip']) echo "    <span id=\"tt_{$newButton['buttonname']}\" class=\"tooltip below\">{$newButton['buttontooltip']}</span>\n";
      }
    } 
?>
</p>
</div>

<div id="dialog_email">
  <p>
    <label for="email_template_id"><?=_("Use template")?>:</label>
<?php
    echo $this->getEmailTemplateListHTML($DB); 
?>
  </p>
  <p>
    <label for="email_subject"><?=_("Subject")?>:</label>
    <input type="text" id="email_subject" name="email_subject" value="">
  </p>
  <p>
    <label for="email_subject"><?=_("Attachment")?>:</label>
    <input type="file" id="email_attachment" name="email_attachment" value="">
  </p>
  <p>
    <label for="email_text"><?=_("Email text")?>:</label>
    <textarea id="email_text" name="email_text"></textarea>
      <span class="form_additional_info"><b><?=_("Variables")?>:</b> $salutation$, $givenname$, $familyname$</span>
  </p>
  <p id="email_warning"></p>
  <input type="submit" id="kdb_send_email" name="kdb_send_email" value="<?=_("Send email")?>" onClick="if(kdbActionDo('ViewDataUI', 'kdb_form_view', 'dialog_email')) return false;">
  <input type="submit" id="kdb_email_cancel" name="kdb_email_cancel" value="<?=_("Cancel")?>" onClick="kdbCancelAction('ViewDataUI', 'kdb_form_view', 'dialog_email'); return false;">
  
  <p>
    <label for="email_template_name"><?=_("Template name")?>:</label>
    <input type="text" id="email_template_name" name="email_template_name" value="">
    <input type="submit" id="kdb_save" name="kdb_save" value="<?=_("Save template")?>" onClick="kdbSaveEmailTemplate('ViewDataUI', 'kdb_form_view'); return false;">
    <input type="submit" id="kdb_delete" name="kdb_delete" value="<?=_("Delete template")?>" onClick="kdbDeleteEmailTemplate('ViewDataUI', 'kdb_form_view'); return false;">
  </p>
</div>

<?php
    if(!$this->eventId && $_SESSION['admin']['login']!="root")
      echo "<p>"._("No event selected.")."</p>\n";
    else
      echo $this->getViewTable($DB);

    echo $this->resultPagination();
?>
<div id="dialog_delete" style="display:none;">
  <input type="submit" id="kdb_delete2" name="kdb_delete" value="<?=_("Really Delete")?>" onClick="kdbActionDo('ViewDataUI', 'kdb_form_view', 'dialog_delete'); return false;">
  <input type="submit" id="kdb_delete_cancel" name="kdb_delete_cancel" value="<?=_("Cancel")?>" onClick="kdbCancelAction('ViewDataUI', 'kdb_form_view', 'dialog_delete'); return false;">
</div>

<div id="dialog_merge" style="display:none;">
  <p>
    <label for="merge_with_id"><?=_("Merge with user id")?>:</label>
    <select id="merge_with_id" name="merge_with_id">
      <option></option>
    </select>
  </p>
  <input type="submit" id="kdb_merge2" name="kdb_merge" value="<?=_("Really Merge")?>" onClick="kdbActionDo('ViewDataUI', 'kdb_form_view', 'dialog_merge'); return false;">
  <input type="submit" id="kdb_merge_cancel" name="kdb_merge_cancel" value="<?=_("Cancel")?>" onClick="kdbCancelAction('ViewDataUI', 'kdb_form_view', 'dialog_merge'); return false;">
</div>

<div class="form_controls_bottom">
<p>
<?php 
    if($this->view->edit_user_data) {
?>
  <input type="submit" id="kdb_save2" name="kdb_save" value="<?=_("Save")?>" onClick="kdbSaveEntry('ViewDataUI', 'save', 'kdb_form_view', refreshHiddenColumns); return false;">
  <input type="submit" id="kdb_delete_selected2" name="kdb_delete" value="<?=_("Delete")?>" onClick="kdbActionConfirmation('ViewDataUI', 'kdb_form_view', 'dialog_delete'); return false;">
  <input type="submit" id="kdb_merge_selected2" name="kdb_merge" value="<?=_("Merge")?>" onClick="kdbActionConfirmation('ViewDataUI', 'kdb_form_view', 'dialog_merge'); return false;">
<?php
    } 
?>
  <input type="submit" id="kdb_email2" name="kdb_email" value="<?=_("Write email")?>" onClick="kdbActionConfirmation('ViewDataUI', 'kdb_form_view', 'dialog_email'); return false;">
</p>
</div>
</form>

<span id="tt_filter" class="tooltip"><?= _("Wildcards:<br><u>*</u> any number of characters,<br><u>?</u> any single character.<br>Regular expressions must be enclosed in '/' (e.g. '/[a-z]+/').<br>Empty with <u>=&quot;&quot</u> or <u>=null</u><br>Not empty with <u>&lt;&gt;&quot;&quot</u> or <u>&lt;&gt;null</u>") ?></span>

<?php 
    if($this->view->edit_user_data) {
?>
<div id="dialog_view" class="dialog_container">
<div class="dialog_columns viewui">
  <div class="tab active" onClick="$('#dialog_view .tab').removeClass('active'); $(this).addClass('active'); $('#dialog_view .dialog').hide();  $('#dialog_view #dialog_user').show();"><?=_("User")?></div>
<?php
      if(isset($_SESSION['event']))
        echo "  <div class=\"tab\" onClick=\"$('#dialog_view .tab').removeClass('active'); $(this).addClass('active'); $('#dialog_view .dialog').hide();  $('#dialog_view #dialog_booking').show();\">"._("Booking")."</div>\n";
?>
  <div class="close_dialog" onClick="$('#dialog_view').hide();"></div>
  <div class="dialog" id="dialog_user">
<?php
      $user_ui = new UserUI($DB);
      $user_ui->getHTMLView($DB); 
?>
  </div>
<?php
      if(isset($_SESSION['event'])) {
?>
  <div class="dialog" id="dialog_booking" style="display:none;">
<?php
        if($this->eventId) {
          $booking_ui = new UserBookingUI($DB);
          $booking_ui->getHTMLView($DB);
        } 
?>
  </div>
<?php
      }
?>
</div>
</div>
<?php
    }
  }
}

?>
