<?php
require_once("icwfData.inc.php");
require_once("FormValidation.inc.php");

class UserBooking implements icwfData {
  private static $noExport = array("form");
  private static $notInArray = array("validation", "form");
  private $validation;
  public $form;
  
  private $booking_id;
  private $user_id;
  private $event_id;
  private $booking_date;
  private $bookedOptions;
  
  private $payment_received;

  /////////////////////////////////////////////////////////////////////////////////////////////////
  // initialize
  private function initialize() {
    foreach ($this as $key => $value)
      if($key == "bookedOptions")
        $this->$key = array();
      else
        $this->$key = null;
  }
  
  public function __construct(FormValidation $validation=null) {
    $this->initialize();
    
    $this->form = new EventForm();
    
    if(!$validation)
      $this->validation = new FormValidation(array(), array());
    else
      $this->validation = $validation;
    
    if(isset($_SESSION['user']))
      $this->user_id = $_SESSION['user']->user_id;
  }
  
  /////////////////////////////////////////////////////////////////////////////////////////////////
  // get values
  public function __get($name) {
    if($name == "id") $name = "booking_id";
    if (array_key_exists($name, get_class_vars(get_class())) && !in_array($name, self::$noExport))
      return $this->$name;
      
    return null;
  }

  public function getIDName() {
    return "booking_id";
  }
  
  public function toArray() {
    $array = array();
    foreach ($this as $key => $value) {
      if(!in_array($key, self::$notInArray)) {
        if($key=="bookedOptions") {
          foreach($value as $optionId=>$option) {
            $array["event_option[".$option->event_option_id."]"] = $option->getValue(1);
            $array["waitlist_event_option[".$option->event_option_id."]"] = $option->waitlist;
          }
        }
        else
          $array[$key] = $value;
      }
    }
    return $array;
  }
  
  public function getPostBackData() {
    return array($this->getIDName()=>$this->id);
  }
  
  public function isWaitlisted() {
    foreach ($this->bookedOptions as $bookedOption)
      if($bookedOption->waitlist)
        return true;
    
    return false;
  }
  
  /////////////////////////////////////////////////////////////////////////////////////////////////
  // set values
  public function setUserId($userId) {
    $this->user_id = $userId;
  }
  
  public function setEventId($DB, $eventId) {
    $this->form->setEventId($eventId);
    $this->form->readData($DB);
    $this->event_id = $eventId;
  }

  public function unsetBookingId() {
    if(isset($_SESSION['admin']))
      $this->booking_id = null;
  } 
  
  public function setValidation(FormValidation $validation) {
    $this->validation = $validation;
  }
  
  public function fromArray($array, $force=false) {
    if(!isset($_SESSION['admin'])) $force=false;
    foreach ($this as $key => $value) {
      switch ($key) {
        case "validation":
        case "booking_date":
        case "payment_received":
        case "form":
          break;
        
        case "user_id":
          if(!isset($_SESSION["user"])) {
            if(isset($array[$key]) && !is_array($array[$key])) { 
              if(preg_match("~^[0-9]+$~", $array[$key]))
                $this->$key = $array[$key];
              else
                $this->$key = null;
            }
            else if(!isset($array[$key]))
              $this->$key = null;
          }
          break;
        case "event_id":
          if(!isset($_SESSION["event"])) {
            if(isset($array[$key]) && preg_match("~^[0-9]+$~", $array[$key]))
              $this->$key = $array[$key];
            else
              $this->$key = null;
          }
          break;
        case "booking_id":
          if(isset($array[$key]) && preg_match("~^[0-9]+$~", $array[$key]))
            $this->$key = $array[$key];
          else
            $this->$key = null;
          break;
            
        case "bookedOptions":
          foreach ($this->form->eventForm as $groupId => $group) {
            foreach ($group->eventOptions as $optionId => $option) {
              if(!$this->booking_id || ($group->group_type==OPTION_GROUP_ADDINFO && $group->allow_changes) || $force) {
                $bookingOption = new UserBookingOption();
                switch($group->group_type) {
                  case OPTION_GROUP_FEE:
                    if ($group->exclusive) {
                      $elementName = "event_option_g";
                      $elementIdx = $groupId;
                    }
                    else {
                      $elementName = "event_option";
                      $elementIdx = $option->event_option_id;
                    }
                    if(isset($array[$elementName][$elementIdx]) && $option->feeExists($array[$elementName][$elementIdx])) {
                      if($option->person_limit>0 && $option->person_limit<=$option->no_of_bookings && !$_SESSION['event']->enable_waitlist && !$force)
                        continue 2;
                      $bookingOption->event_fee_id = $array[$elementName][$elementIdx];
                      if($force)
                        $bookingOption->waitlist = ((isset($array["waitlist_".$elementName][$elementIdx]) && $array["waitlist_".$elementName][$elementIdx]) ? 1 : 0);
                      else
                        $bookingOption->waitlist = (($option->person_limit>0 && $option->person_limit<=$option->no_of_bookings) ? 1 : 0);
                    }
                    else if ($force)
                      $bookingOption->event_fee_id = null;
                    else
                      continue 2;
                      
                    $bookingOption->changeAllowed = 0;
                    break;
                  case OPTION_GROUP_ADDINFO:
                    switch($option->option_type) {
                      case EVENT_OPTION_TEXT:  
                      case EVENT_OPTION_TEXT_MULTILINE:
                        $bookingOption->value_str = $array["event_option"][$option->event_option_id];
                        break;
                      case EVENT_OPTION_DATE:
                        if($array["event_option"][$option->event_option_id])
                          $bookingOption->value_date = CWFTools::dateLocal2ISO($array["event_option"][$option->event_option_id], getenv("LC_ALL"));
                        else
                          $bookingOption->value_date = null;
                        break;
                      case EVENT_OPTION_NUMBER_INT:
                        if($array["event_option"][$option->event_option_id]!=="")
                          $bookingOption->value_num = $array["event_option"][$option->event_option_id];
                        else
                          $bookingOption->value_num = null;
                        break;
                      case EVENT_OPTION_NUMBER_DEC:
                        if($array["event_option"][$option->event_option_id]!=="")
                          $bookingOption->value_num = CWFTools::string2Float($array["event_option"][$option->event_option_id], getenv("LC_ALL"));
                        else
                          $bookingOption->value_num = null;
                        break;
                    }
                    $bookingOption->event_option_type = $option->option_type;
                    $bookingOption->changeAllowed = $group->allow_changes;
                    break;
                }
                $bookingOption->event_group_type = $group->group_type;
                $bookingOption->event_option_id = $option->event_option_id;
                
                $this->bookedOptions[$option->event_option_id] = $bookingOption;
              }
            }
          }
          break;
          
        default:
          $this->$key = null;
          break;
      }
    }
    return $this->validation->validate($array);
  }

  /////////////////////////////////////////////////////////////////////////////////////////////////
  // database operations - read
  public function readData(DbConnection $DB, $conditions=null) {
    if(is_array($conditions)) {
      $whereConditions = array();
      foreach ($conditions as $key => $value)
        if(array_key_exists($key, get_class_vars(get_class())))
          $whereConditions[] = "$key=$value";
      $conditions =  implode(" AND ", $whereConditions);
    }
    if(!$conditions) {
      $conditions = "user_id='{$this->user_id}' AND event_id='{$this->event_id}'";
    }
    $conditions .= " AND deleted=0";
    
    $result = $DB->query("SELECT ub.*, ". 
                         "  (SELECT payment_received FROM user_booking_check ubc WHERE ubc.booking_id=ub.booking_id) payment_received ".
                         "FROM user_booking ub ".
                         "WHERE {$conditions};");
    if(!$DB->errno() && $DB->num_rows($result)>0) {
      $booking = $DB->fetch_array($result);
      foreach ($booking as $property => $value) {
        if(property_exists($this, $property)) {
          switch ($property) {
            default:
              $this->$property = $value;
              break;
          }
        }
      }

      $result = $DB->query("SELECT * FROM user_booking_options WHERE deleted=0 AND booking_id='{$this->booking_id}';");
      while ($bookingOption = $DB->fetch_array($result)) {
        $bookedOption = new UserBookingOption();
        $bookedOption->fromArray($bookingOption);
        
        $this->bookedOptions[$bookedOption->event_option_id] = $bookedOption;
      }
    }

    return array($DB->errno(), $DB->error());
  }
  
  static public function readList(DbConnection $DB) {
    return array();
  }
  
  static public function getFinances($DB) {
    $sql = "SELECT sum(
              (SELECT sum(ef.event_fee)
               FROM user_booking_options ubo
                 JOIN event_fees ef ON (ef.event_fee_id=ubo.event_fee_id)
                 JOIN event_options eo ON (eo.event_option_id=ef.event_option_id)
                 JOIN event_option_groups eog ON (eog.event_option_group_id=eo.event_option_group_id)
               WHERE ubo.booking_id=ub.booking_id
                 AND ubo.deleted=0
                 AND ubo.waitlist=0
                 AND eog.group_type='".OPTION_GROUP_FEE."'
              ) - ifnull(ubc.discount, 0)) as fee_sum, ifnull(ubc.payment_received, 0) as payment_received
            FROM user_booking ub
              LEFT JOIN user_booking_check ubc ON (ubc.booking_id=ub.booking_id)
            WHERE ub.event_id='{$_SESSION['event']->event_id}'
              AND ub.deleted=0
            GROUP BY ubc.payment_received";
    $result = $DB->query($sql);
    
    $finances = array("received"=>0, "owing"=>0);
    while($row = $DB->fetch_array($result)) {
      if ($row['payment_received'])
        $finances["received"] += $row['fee_sum'];
      else
        $finances["owing"] += $row['fee_sum'];
    }
    
    return $finances;
  }
  
  static public function getBookingsByOption($DB) {
    $sql = "SELECT eo.event_option, 
                   sum(if(ubo.waitlist=0, 1, 0)) as count,
                   sum(if(ubo.waitlist>0, 1, 0)) as count_waitlisted,
                   ifnull(sum(ubc.payment_received), 0) as count_paid,
                   sum(ef.event_fee*IF(ubo.booking_id, 1, 0)) AS fee_sum,
                   eo.person_limit
            FROM event_option_groups eog
              JOIN event_options eo ON (eo.event_option_group_id=eog.event_option_group_id)
              JOIN event_fees ef ON (ef.event_option_id=eo.event_option_id)
              LEFT JOIN user_booking_options ubo ON (ubo.event_fee_id=ef.event_fee_id AND ubo.deleted=0)
              LEFT JOIN user_booking ub ON (ub.booking_id=ubo.booking_id AND ub.deleted=0)
              LEFT JOIN user_booking_check ubc ON (ubc.booking_id=ub.booking_id)
            WHERE eog.event_id='{$_SESSION['event']->event_id}'
              AND eog.group_type='".OPTION_GROUP_FEE."'
            GROUP BY eo.event_option_id
            ORDER BY eog.event_option_group_id, eo.event_option_id;";
    $result = $DB->query($sql);

    $bookings = array();
    while($booking = $DB->fetch_array($result)) $bookings[] = $booking;
    
    return $bookings;
  }
  
  /////////////////////////////////////////////////////////////////////////////////////////////////
  // database operations - write
  public function writeData(DbConnection $DB, $force=false) {
    if(!isset($_SESSION['admin'])) $force=false;
    $isChange = false;
    if(isset($this->booking_id) && $this->booking_id) {
      $result = null;
      $isChange = true;
    }
    else if($this->user_id && $this->event_id) {
      $result = $DB->query("INSERT INTO user_booking(
          user_id,
          event_id,
          booking_date,
          deleted
        ) VALUES(
          '{$this->user_id}',
          '{$this->event_id}',
          NOW(),
          0
        );");
      
      $this->booking_id = $DB->insert_id();
      $logEntry = "Booking for user {$this->user_id} and event {$this->event_id} added (id {$this->booking_id}).";
    }
    else
      return array(1, _("User id or event id not set."));

    if($this->booking_id) {
      foreach ($this->bookedOptions as &$option) {
        if(!$isChange || $option->changeAllowed || $force) {
          $option->booking_id = $this->booking_id;
          list($errno, $error) = $option->writeData($DB, $force);
          if ($errno) return array($errno, $error);
        }
      }
      $logEntry = "Booking '{$this->booking_id}' changed (user {$this->user_id}, event {$this->event_id}).";
    }
    if(!$DB->errno()) {
      if(isset($_SESSION['admin'])) KDbLog::writeLog($DB, $logEntry);
      return array(0, "");
    }
    else
      return array($DB->errno(), $DB->error());
  }

  static public function unDeleteData(DbConnection $DB, $id) {
    if(preg_match("~^[0-9]+$~", $id))
      $condition = "booking_id='{$id}'";
    else
      $condition = $id;
      
    $booking_ids = $DB->result($DB->query("SELECT group_concat(booking_id SEPARATOR ', ') FROM user_booking WHERE $condition;"), 0, 0);
    $DB->query("UPDATE user_booking SET deleted=0 WHERE $condition;");
    
    if(!$DB->errno() && $DB->affected_rows()>0) {
      KDbLog::writeLog($DB, "Booking(s) restored ({$condition}).");
      UserBookingOption::unDeleteData($DB, "booking_id IN ({$booking_ids})");
      return array(0, "");
    }
    else 
      return array($DB->errno(), $DB->error());
  }

  static public function deleteData(DbConnection $DB, $id, $reallyDelete=0) {
    if(preg_match("~^[0-9]+$~", $id))
      $condition = "booking_id='{$id}'";
    else
      $condition = $id;
      
    $booking_ids = $DB->result($DB->query("SELECT group_concat(booking_id SEPARATOR ', ') FROM user_booking WHERE $condition;"), 0, 0);
    if($reallyDelete)
      $DB->query("DELETE FROM user_booking WHERE $condition;");
    else
      $DB->query("UPDATE user_booking SET deleted=1 WHERE $condition;");
    
    if(!$DB->errno() && $DB->affected_rows()>0) {
      KDbLog::writeLog($DB, "Booking(s) ".($reallyDelete ? "finally " : "")."deleted ({$condition}).");
      UserBookingOption::deleteData($DB, "booking_id IN ({$booking_ids})", $reallyDelete);
      if($reallyDelete) UserBookingCheck::deleteData($DB, "booking_id IN ({$booking_ids})");
      return array(0, "");
    }
    else 
      return array($DB->errno(), $DB->error());
  }

  /////////////////////////////////////////////////////////////////////////////////////////////////
  // database operations - misc
  static public function createTable(DbConnection $DB) {
    $result = $DB->query("
      CREATE TABLE user_booking (
        booking_id int unsigned NOT NULL AUTO_INCREMENT PRIMARY KEY,
        user_id int unsigned,
        event_id int unsigned,
        booking_date datetime,
        deleted tinyint,
        INDEX user_id (user_id),
        INDEX event_id (event_id)
        );");
    if ($result) { echo _("&nbsp;&nbsp;- creating table").": 'user_booking'.<br>\n"; }
    else { echo _("&nbsp;&nbsp;<b>- Error when creating table</b>")." 'user_booking'<br>\n"; $error = 1; }
  }
  
  /////////////////////////////////////////////////////////////////////////////////////////////////
  // output of formatted data
  static public function getProperties($property=null) {
    $properties = array(
      "booked_options"=>_("Booked options"),
      "grandtotal"=>_("Grand total")
    );
    
    if(is_array($property)) {
      $newProperties = array();
      foreach($property as $id=>$p)
        if(isset($properties[$p])) $newProperties[$id] = $properties[$p];
      return $newProperties;
    }
    else if($property) {
      $property = preg_replace('~^([^.]+\.)?([^.]+)$~', '\2', $property);
      return ((isset($properties[$property])) ? $properties[$property] : null);
    }
    else
      return $properties;
  }

  static public function getPropertySelect($columnNames, $addTableName="") {
    if(is_array($columnNames)) {
      foreach($columnNames as &$columnName)
        $columnName = self::getPropertySelect($columnName, $addTableName);
    }
    else {
      $property = preg_replace('~^([^.]+\.)?([^.]+)$~', '\2', $columnNames);
      if(!in_array($property, array("bookey_options", "grandtotal"))) return $columnNames;

      $columnNames = $addTableName . $property;
      switch ($property) {
        case "booked_options":
          break;
        case "grandtotal":
          break;
      }
    }
    return $columnNames; 
  }
  
  public function getFormData() {
    $formData = array();
    $formAddInfo = "";
    foreach ($this->form->eventForm as $groupId => $group) {
      foreach ($group->eventOptions as $optionId => $option) {
        if(isset($this->bookedOptions[$option->event_option_id]) && $this->bookedOptions[$option->event_option_id]->getValue()) {
          switch($group->group_type) {
            case OPTION_GROUP_FEE:
              $formData[] = $option->event_option;
              break;
            case OPTION_GROUP_ADDINFO:
              $formAddInfo .= $option->event_option . ": " . $this->bookedOptions[$option->event_option_id]->getValue(1) . "\n";
              break;
          }
        }
      }
    }
    $formDataText = _("Booked program").": ".implode(", ", $formData).($formAddInfo ? "\n".$formAddInfo : "");
    
    return $formDataText;
  }
  
  public function getConfirmationText(DbConnection $DB, $confirmationText) {
    $fees = array();
    foreach($this->bookedOptions as $optionId => $option) {
      if($option->event_fee_id) $fees[] = $option->event_fee_id;
    }
    $sqlBookedProgram = "SELECT group_concat(eo.event_option ORDER BY eo.event_option_id SEPARATOR ', ')
      FROM event_options eo
        JOIN event_fees ef ON ef.event_option_id=eo.event_option_id
      WHERE ef.event_fee_id IN ('".implode("','", $fees)."');";
    $sqlGrandTotal = "SELECT sum(ef.event_fee)
      FROM event_fees ef
      WHERE ef.event_fee_id IN ('".implode("','", $fees)."');";
    $confirmationText = str_replace("\$booked_program\$", $DB->result($DB->query($sqlBookedProgram), 0, 0), $confirmationText);
    $confirmationText = str_replace("\$grandtotal\$", $DB->result($DB->query($sqlGrandTotal), 0, 0), $confirmationText);
    
    return $confirmationText;
  }
}
?>
